<?php

namespace QUI\RabbitMQServer;

use DateTime;
use DateTimeImmutable;
use PCSG\Tasks\TaskSerializerInterface;
use PCSG\Tasks\WorkerTask;
use PCSG\Tasks\WorkerTaskDataInterface;
use QUI;
use QUI\QueueManager\QueueJob;
use QUI\QueueManager\QueueJobUUID;
use QUI\RabbitMQServer\Interface\TaskSchedulerInterface;
use QUI\Utils\Uuid;

readonly class TaskScheduler implements TaskSchedulerInterface
{
    /**
     * @param Server $server
     * @param TaskSerializerInterface $taskSerializer
     */
    public function __construct(
        private Server $server,
        private TaskSerializerInterface $taskSerializer
    ) {
    }

    /**
     * Create a queue job for a (QUIQQER-independent) task worker and queue it.
     *
     * @param WorkerTaskDataInterface $taskData
     * @param string $logLevel - PSR-3 log level
     * @param DateTimeImmutable|null $earliestQueueDate (optional) - If omitted, the job will be queued at the earliest
     * @param int|null $priority (optional) - default: 1 (standard priority)
     * @return QueueJobUUID - Worker task / queue job UUID
     *
     * @throws QUI\Exception
     * @throws \DateMalformedStringException
     */
    public function queueWorkerTask(
        WorkerTaskDataInterface $taskData,
        string $logLevel,
        ?DateTimeImmutable $earliestQueueDate = null,
        ?int $priority = 1
    ): QueueJobUUID {
        $uuid = Uuid::get();

        $workerTask = new WorkerTask(
            $uuid,
            $taskData,
            $logLevel,
            $earliestQueueDate
        );

        $queueJob = $this->createQueueJobFromWorkerTask($workerTask);

        $earliestQueueDateJob = new DateTime(
            $earliestQueueDate->format('Y-m-d H:i:s'),
            $earliestQueueDate->getTimezone()
        );

        $queueJob->jobAttributes->earliestQueueDate = $earliestQueueDateJob;
        $queueJob->jobAttributes->priority = $priority;

        return $this->server->queueJob($queueJob);
    }

    /**
     * @param WorkerTask $workerTask
     * @return QueueJob
     */
    public function createQueueJobFromWorkerTask(WorkerTask $workerTask): QueueJob
    {
        $queueJob = new QueueJob(
            new QueueJobUUID($workerTask->uuid)
        );

        $queueJob->data->set(
            JobDataKey::WORKER_TASK_SERIALIZED->value,
            $this->taskSerializer->serializeWorkerTaskForQueue($workerTask)
        );

        $queueJob->data->set(
            JobDataKey::WORKER_TASK_DATA_CLASS->value,
            $workerTask->taskData::class
        );

        return $queueJob;
    }
}
